@@ -1,6 +1,6 @@
 /* Mailnag - GNOME-Shell extension frontend
 *
-* Copyright 2013 - 2016 Patrick Ulbrich <zulu99@gmx.net>
+* Copyright 2013 - 2020 Patrick Ulbrich <zulu99@gmx.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,10 @@
 * MA 02110-1301, USA.
 */
 
-const St = imports.gi.St;
-const Clutter = imports.gi.Clutter;
+const { Clutter, St, Pango, GObject } = imports.gi;
 const Util = imports.misc.util;
-const Pango = imports.gi.Pango;
 const PopupMenu = imports.ui.popupMenu;
 const PanelMenu = imports.ui.panelMenu;
-const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const ExtensionUtils = imports.misc.extensionUtils;
 const Me = ExtensionUtils.getCurrentExtension();
@@ -34,23 +31,22 @@ const Opts = Me.imports.opts;
 const INDICATOR_ICON	= 'mail-unread-symbolic'
 const INACTIVE_ITEM		= { reactive: true, can_focus: false, activate: false, hover: false };
  
-const IndicatorMailMenuItem = new Lang.Class({
-	Name: 'IndicatorMailMenuItem',
-	Extends: PopupMenu.PopupBaseMenuItem,
-
-	_init: function(mail, avatars, avatarSize, showDates, extension) {
-		this.parent(INACTIVE_ITEM);
+var IndicatorMailMenuItem = GObject.registerClass(
+class IndicatorMailMenuItem extends PopupMenu.PopupBaseMenuItem {
+	_init(mail, avatars, avatarSize, showDates, extension) {
+		super._init(INACTIVE_ITEM);
+		
 		this._extension = extension;
 		this._mailID = null;
 		this._dateLabel = null;
 		this._iconBin = null;
 		this._closeButton = null;
 		
-		let [sender, size]		= mail['sender_name'].get_string();
-		let [senderAddr, size]	= mail['sender_addr'].get_string();
-		let [subject, size]		= mail['subject'].get_string();
-		let [mailID, size]		= mail['id'].get_string();
-		let datetime			= mail['datetime'].get_int32();
+		let [sender, ]		= mail['sender_name'].get_string();
+		let [senderAddr, ]	= mail['sender_addr'].get_string();
+		let [subject, ]		= mail['subject'].get_string();
+		let [mailID, ]		= mail['id'].get_string();
+		let datetime		= mail['datetime'].get_int32();
 		
 		this._mailID = mailID;
 		
@@ -75,13 +71,14 @@ const IndicatorMailMenuItem = new Lang.Class({
 		hbox2.add(senderLabel);
 		
 		if (showDates) {
-			this._dateLabel = new St.Label({ text: Util.formatTime(new Date(datetime * 1000)), style_class: 'date-label' });
-			hbox2.add(this._dateLabel, { expand: true, x_fill: false, x_align: St.Align.END });
+			this._dateLabel = new St.Label({ text: Util.formatTime(new Date(datetime * 1000)), style_class: 'mailnag-date-label', 
+											 y_expand: true, x_expand: false, x_align: Clutter.ActorAlign.END });
+			hbox2.add(this._dateLabel);
 		}
 		
 		this._closeButton = new St.Button({ reactive: true, can_focus: true, visible: false, track_hover: true });
 		
-		this._closeButton.connect('clicked', function() {
+		this._closeButton.connect('clicked', () => {
 			extension.markMailAsRead(mailID);
 		});
 		
@@ -100,40 +97,40 @@ const IndicatorMailMenuItem = new Lang.Class({
 			this._iconBin = new St.Bin({ style_class: 'avatar',
 								   style: 'background-image: url("%s")'.format(avatarFile),
 								   width: avatarSize, height: avatarSize,
-								   x_fill: true,
-								   y_fill: true });
+								   x_expand: true,
+								   y_expand: true });
 			hbox.add(this._iconBin);
 		} else {
 			/*hbox.add(new St.Icon({ icon_name: 'avatar-default', 
 											  icon_size: avatarSize }));*/
 		}
 
-		hbox.connect('button-release-event', Lang.bind(this, this._onButtonReleaseEvent));
-		hbox.connect('touch-event', Lang.bind(this, this._onTouchEvent));
-		hbox.connect('key-press-event', Lang.bind(this, this._onKeyPressEvent));
-		hbox.connect('notify::hover', Lang.bind(this, this._onHover));
+		hbox.connect('button-release-event', this._onButtonReleaseEvent.bind(this));
+		hbox.connect('touch-event', this._onTouchEvent.bind(this));
+		hbox.connect('key-press-event', this._onKeyPressEvent.bind(this));
+		hbox.connect('notify::hover', this._onHover.bind(this));
 		
 		hbox.isMailnagMailItem = true;
 		
-		this.actor.add_child(hbox);
-	},
+		this.add_child(hbox);
+	}
 	
-	_onButtonReleaseEvent: function (actor, event) {
+	_onButtonReleaseEvent(actor, event) {
 		Utils.openDefaultMailReader();
 		this.activate(event);
 		return Clutter.EVENT_STOP;
-	},
+	}
 
-	_onTouchEvent: function (actor, event) {
+	_onTouchEvent(actor, event) {
 		if (event.type() == Clutter.EventType.TOUCH_END) {
 			Utils.openDefaultMailReader();
 			this.activate(event);
 			return Clutter.EVENT_STOP;
 		}
 		return Clutter.EVENT_PROPAGATE;
-	},
+	}
 
-	_onKeyPressEvent: function (actor, event) {
+	_onKeyPressEvent(actor, event) {
 		let symbol = event.get_key_symbol();
 
 		if (symbol == Clutter.KEY_space || symbol == Clutter.KEY_Return) {
@@ -145,9 +142,9 @@ const IndicatorMailMenuItem = new Lang.Class({
 			return Clutter.EVENT_STOP;
 		}
 		return Clutter.EVENT_PROPAGATE;
-	},
+	}
 	
-	_onHover: function(actor) {
+	_onHover(actor) {
 		if (this._dateLabel != null)
 			this._dateLabel.visible = !actor.hover;
 		
@@ -158,12 +155,11 @@ const IndicatorMailMenuItem = new Lang.Class({
 	}
 });
 
-const MailnagIndicator = new Lang.Class({
-	Name: 'MailnagIndicator',
-	Extends: PanelMenu.Button,
-	
-	_init: function(options, extension) {
-		this.parent(0.0, this.Name);
+var MailnagIndicator = GObject.registerClass(
+class MailnagIndicator extends PanelMenu.Button {
+	_init(options, extension) {
+		super._init(0.0, null, false);
+		
 		this._opts = options;
 		this._extension = extension;
 		
@@ -171,7 +167,7 @@ const MailnagIndicator = new Lang.Class({
 			icon_name: INDICATOR_ICON,
 			style_class: 'system-status-icon'});
 
-		this._iconBin = new St.Bin({ child: this._icon, x_fill: false, y_fill: false });
+		this._iconBin = new St.Bin({ child: this._icon, x_expand: false, y_expand: false });
 		
 		this._counterLabel = new St.Label({ text: "0",
 											x_align: Clutter.ActorAlign.CENTER,
@@ -182,30 +178,26 @@ const MailnagIndicator = new Lang.Class({
 		this._counterBin = new St.Bin({ style_class: 'mailnag-counter',
 										child: this._counterLabel,
 										layout_manager: new Clutter.BinLayout() });
-
-		this._counterBin.connect('style-changed', Lang.bind(this, function() {
-			let themeNode = this._counterBin.get_theme_node();
-			this._counterBin.translation_x = themeNode.get_length('-mailnag-counter-overlap-x');
-			this._counterBin.translation_y = themeNode.get_length('-mailnag-counter-overlap-y');
-		}));
-		
-		this.actor.add_actor(this._iconBin);
-		this.actor.add_actor(this._counterBin);
+	
+		this.add_actor(this._iconBin);
+		this.add_actor(this._counterBin);
 		
 		this.setMails([]);
-	},
+	}
 	
-	_allocate: function(actor, box, flags) {
+	vfunc_allocate(box) {
+		super.vfunc_allocate(box);
+		
 		// the iconBin should fill our entire box
-		this._iconBin.allocate(box, flags);
+		this._iconBin.allocate(box);
 
 		// get the allocation box of the indicator icon
-		let iconBox = this._iconBin.child.get_allocation_box();
+		let iconBox = this._iconBin.child.first_child.get_allocation_box();
 		// create a temporary box for calculating the counter allocation
 		let childBox = new Clutter.ActorBox();
 
 		let [minWidth, minHeight, naturalWidth, naturalHeight] = this._counterBin.get_preferred_size();
-		let direction = this.actor.get_text_direction();
+		let direction = this.get_text_direction();
 
 		if (direction == Clutter.TextDirection.LTR) {
 			// allocate on the right in LTR
@@ -217,13 +209,13 @@ const MailnagIndicator = new Lang.Class({
 			childBox.x2 = childBox.x1 + naturalWidth;
 		}
 
-		childBox.y1 = iconBox.y2 - (naturalHeight / 2);
+		childBox.y1 = iconBox.y2 - (naturalHeight / 2) - 1;
 		childBox.y2 = childBox.y1 + naturalHeight;
 
-		this._counterBin.allocate(childBox, flags);
-    },
+		this._counterBin.allocate(childBox);
+    }
     
-	_updateMenu: function(mails) {
+	_updateMenu(mails) {
 		let item = null;
 		this.menu.removeAll();
 		
@@ -250,31 +242,40 @@ const MailnagIndicator = new Lang.Class({
 			
 			if (this._opts.menuActions & Opts.ACTION_FLAGS.MARK_ALL_AS_READ) {
 				item = new PopupMenu.PopupMenuItem(_("Mark All As Read"));
-				item.connect('activate', Lang.bind(this, function() {
+				item.connect('activate', () => {
 					// We call markAllMailsAsRead() on the mainloop (deferred) 
 					// because it will cause the menu to be rebuilt
 					// (the 'activate' event is closing the menu and 
 					// rebuilding it while it is being closed, somehow 
 					// reopens the menu).
-					Mainloop.idle_add(Lang.bind(this, function() {
+					Mainloop.idle_add(() => {
 						this._extension.markAllMailsAsRead();
 						return false;
-					}));
-				}));
+					});
+				});
 				this.menu.addMenuItem(item);
 			}
 		}
 		
 		if (this._opts.menuActions & Opts.ACTION_FLAGS.CHECK_FOR_MAIL) {
 			item = new PopupMenu.PopupMenuItem(_("Check For Mail"));
-			item.connect('activate', Lang.bind(this, function() {
+			item.connect('activate', () => {
 				this._extension.checkForMails();
-			}));
+			});
 			this.menu.addMenuItem(item);
 		}
 		
 		if (this._opts.menuActions & Opts.ACTION_FLAGS.SETTINGS)
 			this._addSettingsSubmenu(this.menu);
+			
+		if (this._opts.menuActions & Opts.ACTION_FLAGS.QUIT) {
+			item = new PopupMenu.PopupMenuItem(_("Quit"));
+			item.connect('activate', () => {
+				item.actor.reactive = false;
+				this._extension.shutdown();
+			});
+			this.menu.addMenuItem(item);
+		}
 		
 		if (this.menu.isOpen) {
 			if (mails.length == 0) {
@@ -291,21 +292,21 @@ const MailnagIndicator = new Lang.Class({
 		
 				// If the menu is open, set the key-focus on the panel icon
 				// so the focus won't get lost if a mail was removed via the delete key.
-				this.actor.grab_key_focus();
+				this.grab_key_focus();
 			}
 		}
-	},
+	}
 	
-	_addMailItems: function(menu, mails, maxMails) {
+	_addMailItems(menu, mails, maxMails) {
 		for (let i = 0; i < maxMails; i++) {
 			let item = new IndicatorMailMenuItem(mails[i], this._opts.avatars, 
 				this._opts.avatarSize, this._opts.showDates, this._extension);
 
 			menu.addMenuItem(item);
 		}
-	},
+	}
 	
-	_addGroupedMailItems: function(menu, mails, maxMails) {
+	_addGroupedMailItems(menu, mails, maxMails) {
 		//
 		// Group mails by account
 		//
@@ -331,7 +332,7 @@ const MailnagIndicator = new Lang.Class({
 		counter = Math.max(counter, groups.size);
 		
 		while (counter > 0) {
-			for ([k, v] of groups) {
+			for (let [k, v] of groups) {
 				if (v.length > 0) {
 					if (!groupsTrimmed.has(k))
 						groupsTrimmed.set(k, [])
@@ -363,10 +364,11 @@ const MailnagIndicator = new Lang.Class({
 			if (remainingMails.length > 0) {
 				let hbox = new St.BoxLayout({ vertical: false, x_expand: true, style_class: 'generic-box' });
 				let str = "%d/%d".format(mails.length, mails.length + remainingMails.length);
-				let bin = new St.Bin({ style_class: 'overflow-badge', child: new St.Label( { text: str } ) });
+				let bin = new St.Bin({ style_class: 'overflow-badge', child: new St.Label( { text: str } ), 
+									   y_expand: true, x_expand: false, x_align: Clutter.ActorAlign.END });
 				
 				hbox.add(label);
-				hbox.add(bin, { expand: true, x_fill: false, x_align: St.Align.END });
+				hbox.add(bin);
 				item.actor.add_child(hbox);
 			} else {
 				item.actor.add_child(label);
@@ -375,29 +377,29 @@ const MailnagIndicator = new Lang.Class({
 			menu.addMenuItem(item);
 			this._addMailItems(menu, mails, mails.length);
 		}
-	},
+	}
 	
-	_addSettingsSubmenu: function(menu) {
+	_addSettingsSubmenu(menu) {
 		let item = null;
 		let subMenu = new PopupMenu.PopupSubMenuMenuItem(_("Settings"), false);
 		item = new PopupMenu.PopupMenuItem(_("Mailnag Settings"));
-		item.connect('activate', function() {
+		item.connect('activate', () => {
 			Utils.launchApp('mailnag-config.desktop');
 		});
 		
 		subMenu.menu.addMenuItem(item);
 		
 		item = new PopupMenu.PopupMenuItem(_("Extension Settings"));
-		item.connect('activate', function() {
+		item.connect('activate', () => {
 			Util.spawn(['gnome-shell-extension-prefs', 'mailnag@pulb.github.com']);
 		});
 						
 		subMenu.menu.addMenuItem(item);
 		
 		menu.addMenuItem(subMenu);
-	},
+	}
 	
-	setMails: function(mails) {
+	setMails(mails) {
 		let label = mails.length <= 99 ? mails.length.toString() : "...";
 		this._counterLabel.set_text(label);
 
@@ -408,7 +410,7 @@ const MailnagIndicator = new Lang.Class({
 			this._counterBin.visible = false;
 			this._icon.opacity = 130;
 		}
-		
+
 		this._updateMenu(mails);
 	}
 });
